<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Export extends WordPress_Multilingual_Multisite {

	protected $plugin_name;
	protected $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $translation_tables, $current_blog_id ) 
    {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

        $this->translation_tables = $translation_tables;

        $this->current_blog_id = $current_blog_id;
	}

	public function init() 
    {
		global $wordpress_multilingual_multisite;
		$this->options = $wordpress_multilingual_multisite;

		if (!$this->get_option('enable')) {
			return FALSE;
		}

        if (!$this->get_option('postImportExportEnable')) {
            return FALSE;
        }

        add_action('admin_menu', array($this, 'create_exporter_menu'));
    }

    public function create_exporter_menu() {

        add_submenu_page(
            'wordpress_multilingual_multisite_options',
            esc_html__('Export Posts', 'wordpress-multilingual-multisite'),
            esc_html__('Export Posts', 'wordpress-multilingual-multisite'),
            'manage_options',
            'wordpress-multilingual-multisite-exporter',
            array($this, 'settings_page')
        );
    }

    public function settings_page() {

        if( !current_user_can('administrator') )  {
            return false;
        }

        $sites = get_sites();
        $post_stati = get_post_stati();

        $not_allowed_post_types = array(
            "attachment",
            "revision",
            "custom_css",
            "customize_changeset",
            "oembed_cache",
            "user_request",
            "scheduled-action",
            "shop_order",
            "shop_order_refund",
            "shop_coupon",
            "vc4_templates",
            "vc_grid_item",
        );
        $post_types = array_diff(get_post_types(), $not_allowed_post_types);

        ?>
        <div class="wrap">
            <h1><?php echo esc_html__('Export Posts', 'wordpress-multilingual-multisite') ?></h1>

            <form method="post" enctype="multipart/form-data">

                <table class="form-table">       
                    <tr valign="top">
                        <th scope="row"><?php esc_html_e('Source Blog<br>Leave empty to download posts from current site.', 'wordpress-multilingual-multisite') ?></th>
                        <td>
                            <select name="wordpress-multilingual-multisite-export-source-blog" id="source-blog">
                                <option value=""><?php esc_html_e('Select source blog', 'wordpress-multilingual-multisite') ?></option>
                                <?php 
                                foreach ($sites as $site) {
                                    echo '<option value="' . $site->blog_id . '">' . $site->domain . $site->path . ' (ID: ' . $site->blog_id . ')</option>';
                                }
                                ?>
                            </select>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row"><?php esc_html_e('Target Blog<br>Leave empty if you import directly via admin panel of the target blog.', 'wordpress-multilingual-multisite') ?></th>
                        <td>
                            <select name="wordpress-multilingual-multisite-export-target-blog" id="target-blog">
                                <option value=""><?php esc_html_e('Select target blog', 'wordpress-multilingual-multisite') ?></option>
                                <?php 
                                foreach ($sites as $site) {
                                    echo '<option value="' . $site->blog_id . '">' . $site->domain . $site->path . ' (ID: ' . $site->blog_id . ')</option>';
                                }
                                ?>
                            </select>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row"><?php esc_html_e('Post Types<br>', 'wordpress-multilingual-multisite') ?></th>
                        <td>
                            <select name="wordpress-multilingual-multisite-export-post-types" id="post-type" multiple="true">
                                <option value="any"><?php esc_html_e('Any', 'wordpress-multilingual-multisite') ?></option>
                                <?php 
                                foreach ($post_types as $post_type) {
                                    $post_type = esc_attr($post_type);
                                    echo '<option value="' . $post_type . '">' . $post_type . '</option>';
                                }
                                ?>
                            </select>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Post Status<br></th>
                        <td>
                            <select name="wordpress-multilingual-multisite-export-post-status" id="post-status">
                                <option value="any"><?php esc_html_e('Any', 'wordpress-multilingual-multisite') ?></option>
                                <?php 
                                foreach ($post_stati as $post_status) {
                                    $post_type = esc_attr($post_status);
                                    echo '<option value="' . $post_status . '">' . $post_status . '</option>';
                                }
                                ?>
                            </select>
                        </td>
                    </tr>
                </table>

                <p class="submit">
                    <input type="submit" name="wordpress-multilingual-multisite-export" id="submit" class="button button-primary" value="Export">
                </p>
            </form>
        </div>
    <?php 
    }

    /**
     * Add custom ticket metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @param   [type]                       $post_type [description]
     * @param   [type]                       $post      [description]
     */
    public function add_metabox($post_type, $post)
    {
        if(!$this->get_option('postImportExportEnable')) {
            return false;
        }

        add_meta_box('wordpress-multilingual-multisite-import-export', esc_html__('Import Export Posts', 'wordpress-multilingual-multisite'), array($this, 'metabox'), '', 'side', 'high');
    }

    /**
     * Display Metabox metabox
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @return  [type]                       [description]
     */
    public function metabox()
    {
        global $post, $blogs_data;
                
        echo '<div class="wordpress-multilingual-multisite-container">';
            echo '<input type="hidden" name="wordpress-multilingual-multisite-export-post-id" value="' . $post->ID . '">';
            echo '<button type="submit" class="button button-primary" name="wordpress-multilingual-multisite-export" value="Export">Export</button>';
        echo '</div>';
    }

    public function maybe_export()
    {
        if( !current_user_can('administrator') )  {
            return false;
        }

        if(!isset($_POST['wordpress-multilingual-multisite-export'])) {
            return false;
        }

        $data = array(
            'source_blog' => null,
            'target_blog' => null,
            'posts' => array()
        );

        $source_blog = isset($_POST['wordpress-multilingual-multisite-export-source-blog']) && !empty($_POST['wordpress-multilingual-multisite-export-source-blog']) ? $_POST['wordpress-multilingual-multisite-export-source-blog'] : $this->current_blog_id;
        $target_blog = isset($_POST['wordpress-multilingual-multisite-export-target-blog']) && !empty($_POST['wordpress-multilingual-multisite-export-target-blog']) ? $_POST['wordpress-multilingual-multisite-export-target-blog'] : null;

        if(isset($_POST['wordpress-multilingual-multisite-export-post-id']) && !empty($_POST['wordpress-multilingual-multisite-export-post-id'])) {
            $post =  (array) get_post($_POST['wordpress-multilingual-multisite-export-post-id']);
            $post['metas'] = get_post_meta($post['ID']);
            $data['posts'][] = $post;
        } else {            
            $post_types = isset($_POST['wordpress-multilingual-multisite-export-post-types']) && !empty($_POST['wordpress-multilingual-multisite-export-post-types']) ? $_POST['wordpress-multilingual-multisite-export-post-types'] : array();
            $post_status = isset($_POST['wordpress-multilingual-multisite-export-post-status']) && !empty($_POST['wordpress-multilingual-multisite-export-post-status']) ? $_POST['wordpress-multilingual-multisite-export-post-status'] : array();

            if(!empty($source_blog)) {
                switch_to_blog($source_blog);
            }

            $tmp = get_posts( array(
                'posts_per_page'   => -1,
                'post_type'        => $post_types,
                'post_status'      => $post_status,
            ) );

            if(empty($tmp)) {
                die('No posts found');
            }

            foreach ($tmp as $tmp_) {
                $post = (array) $tmp_;
                $post['metas'] = get_post_meta($post['ID']);
                $data['posts'][] = $post;
            }

            if(!empty($source_blog)) {
                restore_current_blog();
            }
        }

        $data['source_blog'] = $source_blog;
        $data['target_blog'] = $target_blog;

        if($this->get_option('postImportExportFormat') == "json") {
            $this->generate_json_export($data);
        } else {
            $this->generate_xml_export($data);
        }
    }

    public function generate_xml_export($data)
    {
        $xml_data = new SimpleXMLElement('<?xml version="1.0"?><data></data>');

        // function call to convert array to xml
        $this->array_to_xml($data, $xml_data);

        //saving generated xml file; 
        $result = $xml_data->asXML();
        header("Content-type: application/xml");
        header("Content-Encoding: UTF-8");
        header("Content-Disposition: attachment; filename=\"export.xml\""); 
        echo $result;
        die();
    }

    public function array_to_xml( $data, &$xml_data )
    {
        foreach( $data as $key => $value ) {
            if( is_numeric($key) ){
                $key = 'item'.$key; //dealing with <0/>..<n/> issues
            }
            if( is_array($value) ) {
                $subnode = $xml_data->addChild($key);
                $this->array_to_xml($value, $subnode);
            } else {
                $xml_data->addChild("$key",htmlspecialchars("$value"));
            }
         }
    }

    public function generate_json_export($data)
    {
        $result = json_encode($data);
        header("Content-type: application/xml");
        header("Content-Encoding: UTF-8");
        header("Content-Disposition: attachment; filename=\"export.json\""); 
        echo $result;
        die();
    }
}