<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Terms_Translations extends WordPress_Multilingual_Multisite {

    protected $plugin_name;
    protected $version;
    protected $current_blog_id;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version, $network_query, $translation_tables, $current_blog_id ) 
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->network_query = $network_query;

        $this->translation_tables = $translation_tables;

        $this->prefix = 'wordpress_multilingual_multisite_';
        $this->current_blog_id = $current_blog_id;

    }

    public function init() 
    {
        global $wordpress_multilingual_multisite;

        $this->options = $wordpress_multilingual_multisite;

        if (!$this->get_option('enable')) {
            return FALSE;
        }

        $taxonomies = get_taxonomies();
        foreach ($taxonomies as $taxonomy) {
            add_filter('manage_edit-' . $taxonomy . '_columns', array($this, 'modify_columns_head'), 10);
            add_filter('manage_' . $taxonomy . '_custom_column', array($this, 'modify_columns_content'), 10, 3);

            add_action( $taxonomy . '_add_form_fields',  array( $this, 'connected' ), 30, 1 );
            add_action( $taxonomy . '_edit_form', array( $this, 'connected' ), 60, 1 );
            add_action( 'edited_' . $taxonomy, array( $this, 'save_connection' ), 10, 2 );
            add_action( 'create_' . $taxonomy, array( $this, 'save_connection' ), 10, 2 );
        }
    }

    public function modify_columns_head($columns)
    {
        global $blogs_data;

        if(!$this->get_option('showConnectedItemsInOverview')) {
            return $columns;
        }

        if(count($blogs_data) > 14) {
            return $columns;
        }

        // $excludedSites = $this->get_option('excludedSites');
        // if($excludedSites && in_array($this->current_blog_id, $excludedSites)) {
        //     return $columns;
        // }

        foreach ($blogs_data as $blog_id => $blog_data) {
            if($blog_id == $this->current_blog_id) {
                continue;
            }
            $columns['blog_' . $blog_id] = '<span class="flag-icon flag-icon-' . strtolower($blog_data['country']) . '"></span><br><span style="display: none;">' . $blog_data['country'] . '-' . $blog_data['language'] . '</span>';
        }

        return $columns;
    }

    public function modify_columns_content($content, $column_name, $term_id) 
    {
        global $blogs_data, $term;

        if(!$this->get_option('showConnectedItemsInOverview')) {
            return $content;
        }

        if(count($blogs_data) > 14) {
            return $content;
        }
        
        // $excludedSites = $this->get_option('excludedSites');
        // if($excludedSites && in_array($this->current_blog_id, $excludedSites)) {
        //     return $content;
        // }

        $this->translation_tables->set_table('terms_translations');
        $translated = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $term_id);        
        
        if(substr($column_name, 0, 5) == "blog_") {
            $blog_id = substr($column_name, 5);

            $taxonomy = isset($_GET['taxonomy']) ? '&taxonomy=' . $_GET['taxonomy'] : '';
            $post_type = isset($_GET['post_type']) ? '&post_type=' . $_GET['post_type'] : '';

            if(isset($translated[$blog_id])) {
                
                $link = get_admin_url($blog_id, 'post.php?action=edit&post=' . $translated[$blog_id]);
                $link = get_admin_url($blog_id, 'term.php?tag_ID=' . $translated[$blog_id]) . $post_type . $taxonomy;
                $html = '<a href="' . $link . '"><i class="fa fa-pen-square fa-2x"></i></a>';
            } else {
                $link = get_admin_url($blog_id, '?wp-trans-copy=term&source_blog=' . $this->current_blog_id . '&source_id=' . $term_id);
                $html = '<a href="' . $link . '"><i class="fa fa-plus-square fa-2x"></i></a>';
            }
            return wp_kses_post($html);
        }

        return $content;
    }

    /**
     * Display Metabox metabox
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @return  [type]                       [description]
     */
    public function connected($taxonomy)
    {
        global $term, $blogs_data;

        if(is_a($taxonomy, 'WP_Term')) {
            $term = $taxonomy;
            $taxonomy = $term->taxonomy;
        } else {
            $term = new stdClass();
            $term->term_id = 0;
        }

        $args = array( 
            'taxonomy' => $taxonomy,
            'hide_empty' => false,
            'sites_not' => $this->current_blog_id,
            'orderby' => 'name',
            'order' => 'ASC' 
        );

        if(strpos($taxonomy, 'pa_') !== FALSE) {
            $terms_by_site =  $this->network_query->get_wc_attribute_terms( $args );
        } else {
            $terms_by_site =  $this->network_query->get_terms( $args );
        }


        
        wp_nonce_field(basename(__FILE__), 'wordpress_multilingual_multisite_meta_nonce');

        $this->translation_tables->set_table('terms_translations');
        $translated = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $term->term_id);

        echo '<div class="wordpress-multilingual-multisite-container">';
            echo '<input type="hidden" name="translated_terms[' . $this->current_blog_id . ']" value="' . $term->term_id . '">';
            echo '<input type="hidden" name="translated_terms[type]" value="' . $taxonomy . '">';

            if(!empty($terms_by_site)) {
                foreach ($terms_by_site as $blog_id => $terms_by_site_terms) {

                    if($this->current_blog_id == $blog_id) {
                        continue;
                    }

                    $country = strtolower($blogs_data[$blog_id]['country']);
                    $options = '<option value="0">None</option>';


                    if(strpos($taxonomy, 'pa_') !== FALSE) {
                        $link = get_admin_url($blog_id, '?wp-trans-copy=term&source_blog=' . $this->current_blog_id . '&source_id=' . $term->term_id);
                    } else {
                        $link = get_admin_url($blog_id, '?wp-trans-copy=term&source_blog=' . $this->current_blog_id . '&source_id=' . $term->term_id);
                    }

                    if(!empty($terms_by_site_terms)) {
                        foreach ($terms_by_site_terms as $terms_by_site_term) {
                            $selected = "";
                            
                            if(strpos($taxonomy, 'pa_') !== FALSE) {
                                $options .= '<optgroup label="' . $terms_by_site_term->attribute_label . '">';
                                foreach ($terms_by_site_term->terms as $wc_term) {

                                    $selected = "";
                                    if(isset($translated[$blog_id]) && !empty($translated[$blog_id]) && $wc_term->term_id == $translated[$blog_id]) {
                                        $selected = 'selected="selected"';
                                        $link = get_admin_url($blog_id, 'term.php?taxonomy=' . $taxonomy . '&tag_ID=' . $wc_term->term_id);
                                    }
                                    $options .= '<option value="' . $wc_term->term_id . '" ' . $selected . '>' . $wc_term->name . ' (ID: ' . $wc_term->term_id . ')' . '</option>';
                                }
                                $options .= '</optgroup>';
                            } else {

                                if(isset($translated[$blog_id]) && !empty($translated[$blog_id]) && $terms_by_site_term->term_id == $translated[$blog_id]) {
                                    $selected = 'selected="selected"';
                                    $link = get_admin_url($blog_id, 'term.php?taxonomy=' . $taxonomy . '&tag_ID=' . $terms_by_site_term->term_id);
                                }
                                $options .= '<option value="' . $terms_by_site_term->term_id . '" ' . $selected . '>' . $terms_by_site_term->name . ' (ID: ' . $terms_by_site_term->term_id . ')' . '</option>';
                            }
                        }
                    }
                    ?>

                    <div class="wordpress-multilingual-multisite-row">
                        <div class="wordpress-multilingual-multisite-col-sm-1">
                            <a href="<?php echo $link ?>"><span class="flag-icon flag-icon-<?php echo $country ?>"></span></a>
                        </div>

                        <div class="wordpress-multilingual-multisite-col-sm-11">

                            <select name="translated_terms[<?php echo $blog_id ?>]" class="wordpress-multilingual-multisite-input-field">
                                <option value=""></option>
                                <?php echo $options ?>
                            </select>
                        </div>
                    </div>
                    <?php
                }
            }
        echo '</div>';
    }

    /**
     * Save Custom Metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @param   [type]                       $post_id [description]
     * @param   [type]                       $post    [description]
     * @return  [type]                                [description]
     */
    public function save_connection($term_id, $tt_id)
    {
        if (!isset($_POST['wordpress_multilingual_multisite_meta_nonce']) || !wp_verify_nonce($_POST['wordpress_multilingual_multisite_meta_nonce'], basename(__FILE__))) {
            return false;
        }

        if(!isset($_POST['translated_terms']) || empty($_POST['translated_terms'])) {
            return flase;
        }

        $translated_terms = $_POST['translated_terms'];
        $translated_terms[$this->current_blog_id] = $term_id;
        
        $this->translation_tables->set_table('terms_translations');
        $exists = $this->translation_tables->get_translated_items_by_translated_items($translated_terms);

        if($exists) {
            $this->translation_tables->update_translated_items($exists['id'], $this->current_blog_id, $term_id, $translated_terms);
        } else {
            $this->translation_tables->insert_translated_items($this->current_blog_id, $term_id, $translated_terms);
        }

        delete_site_transient('wordpress_multilingual_multisite_connected_terms');
    }
}