<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Translator extends WordPress_Multilingual_Multisite {

	protected $plugin_name;
	protected $version;
	protected $current_blog_id;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $current_blog_id) 
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->current_blog_id = $current_blog_id;
	}

	public function init() 
	{

		global $wordpress_multilingual_multisite;
		$this->options = $wordpress_multilingual_multisite;
	}

    /**
     * Save Updated Pages as Pending Reviews
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @param   [type]                       $post_id [description]
     * @param   [type]                       $post    [description]
     * @return  [type]                                [description]
     */
    public function override_post_status($post_id, $post, $update)
    {
        global $post;

        if(!$this->get_option('translatorEnable')) {
        	return false;
        }

        if(!is_a($post, 'WP_Post')) {
            return false;
        }

        if (!current_user_can('edit_post', $post->ID)) {
            return $post->ID;
        }

        if($this->get_current_user_role() !== "translator") {
        	return false;
        }

        if(!$this->get_option('translatorRevision')) {
        	return false;
        }

        $post->post_status = 'pending';
		remove_action('save_post', array($this, 'override_post_status'), 1);
        wp_update_post($post, $post->ID);

    }
    
	/**
	 * add_file_admin_order_meta_box.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	public function add_translator_meta_box() 
	{
        if(!$this->get_option('translatorEnable')) {
        	return false;
        }

        if($this->get_current_user_role() == "translator") {
        	return false;
        }

		$context  = 'side';
		$priority = 'high';
		add_meta_box(
			'assigned_translator',
			esc_html__( 'Assign Translator', 'wordpress-multilingual-multisite' ),
			array( $this, 'create_translator_meta_box' ),
			'',
			$context,
			$priority
		);
	}

	public function create_translator_meta_box() 
	{
		global $post;

		$translators = get_users(array('role' => 'translator'));
		if(empty($translators)) {
			echo esc_html__('No Translators created or assigned to this site.', 'wordpress-multilingual-multisite');
			return false;
		}

		$current_assigned = get_post_meta( $post->ID, 'translator', true );

		echo '<select name="translator" id="translator" style="width: 100%;">';
		echo '<option value="">' . esc_html__('Select Translator', 'wordpress-multilingual-multisite') . '</option>';
		foreach ($translators as $translator) {
			$selected = '';
			if($translator->data->ID == $current_assigned) {
				$selected = 'selected="selected"';
			}
			echo '<option value="' . $translator->data->ID . '" ' . $selected . '>' . esc_attr( $translator->data->display_name ) . '</option>';
		}		
		echo '</select>';
		echo '<button type="submit" class="button button-primary" name="save" value="Update">Update</button>';
	}	

	public function save_translator($post_id, $post, $update)
	{
		global $post;

        if(!$this->get_option('translatorEnable')) {
        	return false;
        }

        if(!is_a($post, 'WP_Post')) {
            return false;
        }
        
        if (!current_user_can('edit_post', $post->ID)) {
            return $post->ID;
        }

        if ($post->post_type == 'revision') {
            return false;
        }

		if(!isset($_POST['translator']) || empty($_POST['translator'])) {
			return false;
		}

		$translator = intval($_POST['translator']);
		$before_translator = intval(get_post_meta($post->ID, 'translator', true));

		if($translator == $before_translator) {
			return false;
		}

		update_post_meta($post->ID, 'translator', $translator);
		
		if(!$this->get_option('translatorEmail')) {
			return false;
		}

		$translator_data = get_userdata($translator);
		if(!empty($translator_data->user_email)) {
			$to = $translator_data->user_email;
			$subject =  esc_html__('New Post Assigned for Translating', 'wordpress-multilingual-multisite') . ' | ' . get_bloginfo( 'name' );
			$body = sprintf( esc_html__('A new post has been assigned for you to translate.<br><br>Please <a href="%s">login here and translate the post here</a>.', 'wordpress-multilingual-multisite'), admin_url('post.php?post=' . $post_id . '&action=edit') );
			$headers = array('Content-Type: text/html; charset=UTF-8');

			wp_mail( $to, $subject, $body, $headers );
		}	
	}

	public function filter_posts_by_translator($query)
	{
		global $pagenow;

        if(!$this->get_option('translatorEnable')) {
        	return false;
        }

		$user = wp_get_current_user();

		if(empty($user) || $user->ID == 0) {
			return false;
		}

        if($this->get_current_user_role() !== "translator") {
        	return false;
        }

		$meta_key_query = array(
			array(
			  'key'     => 'translator',
			  'value'   => $user->ID,
			)
		);
		$query->set( 'meta_query', $meta_key_query );
	}

	public function add_translator_class($classes) 
	{
        if(!$this->get_option('translatorEnable')) {
        	return false;
        }

		$user = wp_get_current_user();
	    $role = ( array ) $user->roles;

        $classes .= ' role-' . $role[0] . ' ';
        return $classes;
	}

	public function remove_menu_pages() {

		global $user_ID;
		
        if(!$this->get_option('translatorEnable')) {
        	return false;
        }

        if($this->get_current_user_role() !== "translator") {
        	return false;
        }

		remove_menu_page( 'vc-welcome' );
		remove_menu_page( 'tools.php' );
		remove_menu_page( 'edit-comments.php' );
		remove_submenu_page( 'edit.php', 'post-new.php' );

		$post_types = get_post_types();
		foreach ($post_types as $post_type) {
			remove_submenu_page( 'edit.php?post_type=' . $post_type, 'post-new.php?post_type=' . $post_type );
		}
	}
}